/* progress.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdio.h>
#include "progress.h"
#include "gui.h"
#include "licon.xpm"

static gint delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Progress*)data)->hide(); return TRUE; }

Progress::Progress()
{
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Progress");
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, TRUE);
	gtk_widget_realize(window);
	window_icon(window, licon_xpm);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete_cb), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_cb), this);

  vbox = gtk_vbox_new (FALSE, 8);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);

  label = gtk_label_new ("Import file %s");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

  progressbar = gtk_progress_bar_new ();
  gtk_widget_show (progressbar);
  gtk_box_pack_start (GTK_BOX (vbox), progressbar, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  button = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
}

Progress::~Progress()
{
}

void Progress::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Progress::hide()
{
	gtk_widget_hide(window);
}

void Progress::setTitle(char *str)
{
  gtk_window_set_title(GTK_WINDOW (window), str);
	gdk_window_set_icon_name(window->window, str);
}

void Progress::setLabel(char *str)
{
	gtk_label_set_text(GTK_LABEL(label), str);
}

void Progress::setValue(gfloat num)
{
	gtk_progress_set_percentage(GTK_PROGRESS(progressbar), num);
}

