/* sessionlist.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "sessionlist.h"

static gint delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((SessionList*)data)->hide(); return TRUE; }
static void select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{
	gchar *text;
	gtk_clist_get_text(GTK_CLIST(clist), row, 0, &text);
	gtk_entry_set_text(GTK_ENTRY(((SessionList*)data)->entry), text);
}

SessionList::SessionList()
{
	window = gtk_dialog_new();
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
	gtk_widget_set_usize(window, 190, 230);
	gtk_container_border_width(GTK_CONTAINER(window), 5);
  gtk_window_set_title (GTK_WINDOW (window), "Session List");
	gtk_widget_realize(window);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete_cb), this);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(delete_cb), this);

  vbox38 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox38);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->vbox), vbox38, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vbox38), 4);

  scrolledwindow17 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow17);
  gtk_box_pack_start (GTK_BOX (vbox38), scrolledwindow17, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow17), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist15 = gtk_clist_new (1);
  gtk_widget_show (clist15);
  gtk_container_add (GTK_CONTAINER (scrolledwindow17), clist15);
  gtk_clist_set_column_width (GTK_CLIST (clist15), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clist15));
	gtk_signal_connect(GTK_OBJECT(clist15), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);

  label88 = gtk_label_new ("Session List");
  gtk_widget_show (label88);
  gtk_clist_set_column_widget (GTK_CLIST (clist15), 0, label88);

  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (vbox38), entry, FALSE, FALSE, 0);

  button_accept = gtk_button_new_with_label ("Accept");
	gtk_widget_show (button_accept);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area), button_accept, TRUE, TRUE, 0);
  gtk_widget_set_usize (button_accept, 60, -2);

  button_cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button_cancel);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area), button_cancel, TRUE, TRUE, 0);
  gtk_widget_set_usize (button_cancel, 60, -2);
}

SessionList::~SessionList()
{
}

void SessionList::show()
{
	gtk_entry_set_text(GTK_ENTRY(entry), "");
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void SessionList::hide()
{
	gtk_widget_hide(window);
}

void SessionList::init(char *datadir)
{
	char buf[256], buf2[192];
	struct dirent *file;
	gchar *data[1];
	SessionName *nm;
	Link *ln;
	DIR *d;
	int row;

	// clear the current session list
	for (ln=names.get_head(); ln!=NULL; ln=ln->next)
	{ nm = (SessionName*) ln->item; delete nm; }
	names.empty();

	// scan the sessions directory and build new session list
	sprintf(buf, "%s/sessions", datadir);
	d = opendir(buf);
	if (!d) return;
	file=readdir(d);
	while(file!=NULL)
	{
		if ((strcmp(file->d_name, ".")!=0) && (strcmp(file->d_name, "..")!=0))
		{
			sprintf(buf2, "%s/%s", datadir, file->d_name);
			nm = new SessionName();
			strcpy(nm->filename, buf2);
			strcpy(nm->label, file->d_name);
			names.tag(nm);
		}
		file=readdir(d);
	}
	closedir(d);

	gtk_clist_freeze(GTK_CLIST(clist15));
	gtk_clist_clear(GTK_CLIST(clist15));

	row = 0;
	for (ln=names.get_head(); ln!=NULL; ln=ln->next)
	{
		nm = (SessionName*) ln->item;
		data[0] = strdup(nm->label);
		gtk_clist_append(GTK_CLIST(clist15), data);
		delete data[0];
		gtk_clist_set_row_data(GTK_CLIST(clist15), row, nm);
		row++;
	}
	gtk_clist_set_sort_column (GTK_CLIST (clist15), 0);
	gtk_clist_sort (GTK_CLIST (clist15));
	gtk_clist_thaw(GTK_CLIST(clist15));
}

void SessionList::setTitle(char *str)
{
  gtk_window_set_title(GTK_WINDOW (window), str);
	gdk_window_set_icon_name(window->window, str);
}

char *SessionList::getName()
{
	return (gtk_entry_get_text(GTK_ENTRY(entry)));
}

