/* process.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <pwd.h>
#include "process.h"
#include "misc.h"
#include "graph.h"

#define BUFSIZE 256

Task::Task()
{
	pid = 0;
	status = 0;
	strcpy(cmnd, "");
	ts = 0;
}

Process::Process()
{
	tid = 0;
}

int Process::create()
{
	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fd)<0)
		return 0;

	pid = fork();
	if (pid == -1)
	{
		return 0;
	}else
	if (pid == 0)
	{
		close(fd[1]);
		child();
		close(fd[0]);
		_exit(0);
	}

	fcntl(fd[1], F_SETFL, O_NONBLOCK);

	return 1;
}

void Process::child()
{
	struct passwd *pwd;
	char datadir[256];
	int i, quit = 0;
	char buf[BUFSIZE];
	char msg[BUFSIZE];
	char str[256];
	char str2[256];
	char *md;
	Graph g;

	md = getenv("NOMAD_DIR");
	if (md!=NULL) { strcpy(datadir, md); } else
	{ pwd = getpwuid(getuid());
		sprintf(datadir, "%s/%s", pwd->pw_dir, DATA_DIR);
	}

	fcntl(fd[0], F_SETFL, O_NONBLOCK);

	while(!quit)
	{
		i = read(fd[0], buf, BUFSIZE);
		if (i>0)
		{
			sprintf(msg, "DONE %s", buf);
			if (starts_with("nomad-traffic", buf))
			{
				sprintf(str, "nice -n 19 %s", buf);
				system(str);
				sscanf(buf, "nomad-traffic %[^ ] %[^\n]", str, str2);
				g.main(datadir, str);
			}else
			if (strcmp(buf, "quit")==0) quit = 1; else
				sprintf(msg, "FAILED %s", buf);
			write(fd[0], msg, strlen(msg)+1);
		}
		usleep(2000000);
	}
}

void Process::send(char *str)
{
	write(fd[1], str, strlen(str)+1);
}

int Process::get(char *str)
{
	int i;
	char buf[BUFSIZE];

	i = read(fd[1], buf, BUFSIZE);
	if (i>0)
	{
		strcpy(str, buf);
		return 1;
	}
	return 0;
}

int Process::check_link()
{
	return (kill(pid, 0));
}

