#include <string.h>
#include <stdlib.h>
#include "filter.h"
#include "filteritem.h"
#include "ipaddress.h"

Filter::Filter()
{
	empty();
}

Filter::~Filter()
{
	empty();
}

Filter::Filter(Filter &orig)
{
	Link *ln;
	FilterItem *in, *in2;

	empty();
	strcpy(name, orig.name);
	default_action = orig.default_action;
	for (ln=orig.filter_items.get_head(); ln!=NULL; ln=ln->next)
	{
		in2 = (FilterItem*) ln->item;
		in = new FilterItem(*in2);
		filter_items.tag(in);
	}
}

/*
void Filter::operator = (const Filter &orig)
{
	Link *ln;
	FilterItem *in, *in2;

	empty();
	strcpy(name, orig.name);
	default_action = orig.default_action;

	for (ln=((List)(orig.filter_items)).get_head(); ln!=NULL; ln=ln->next)
	{
		in2 = (FilterItem*) ln->item;
		in = new FilterItem(*in2);
		filter_items.tag(in);
	}

}
*/

void Filter::empty()
{
	Link *ln;
	FilterItem *in;

	strcpy(name, "");
	default_action = 'd';
	for (ln=filter_items.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (FilterItem*) ln->item;
		if (in!=NULL) delete in;
		ln->item = NULL;
	}
	filter_items.empty();
}

void Filter::print()
{
	Link *ln;
	FilterItem *in;

	printf("Name: %s\n", name);
	printf("Action: %s\n", (default_action == 'd')?"Deny":"Permit");
	for (ln=filter_items.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (FilterItem*) ln->item;
		in->print();
	}
}

void Filter::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	char buf[64];
	Link *ln;
	FilterItem *in;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Filter", NULL);
	if (strcmp(name, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
	xmlSetProp(newxml, (xmlChar*)"default_action",
		(xmlChar*)((default_action=='d')?"Deny":"Permit"));
	for (ln=filter_items.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (FilterItem*) ln->item;
		if (in!=NULL)
			in->savexml(newxml);
	}
}

void Filter::loadxml(xmlNodePtr xmlnode)
{
	xmlChar *buf;
	FilterItem *in;

	empty();
	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "Filter")!=0)
		return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	if (buf!=NULL) { strcpy(name, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"default_action");
	if (buf!=NULL)
	{ default_action = ((strcmp((char*)buf, "Deny")==0)?'d':'p'); free(buf); }

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "FilterItem")==0)
			{
				in = new FilterItem();
				in->loadxml(xmlnode);
				filter_items.tag(in);
			}
		}
		xmlnode = xmlnode->next;
	}
}

int Filter::match(IPAddress ip)
{
	Link *ln;
	FilterItem *in;

	for (ln=filter_items.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (FilterItem*) ln->item;
		if (in!=NULL)
		{
			if (ip.in_range(in->ip1, in->ip2))
				return (in->action=='p');
		}
	}
	return (default_action=='p');
}

